/*********************************************************************
 *
 * Authors: Vincenzo Ciaschini - Vincenzo.Ciaschini@cnaf.infn.it 
 *
 * Copyright (c) 2002, 2003 INFN-CNAF on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or
 * http://www.edg.org/license.html
 *
 * Parts of this code may be based upon or even include verbatim pieces,
 * originally written by other people, in which case the original header
 * follows.
 *
 *********************************************************************/
#include <iterator>
#include <string>

#include <sqlplus.hh>
//#include "sqlwrap.h"


class DBEXC {};
class DBEXC_Connect : public DBEXC {};
class DBEXC_Query   : public DBEXC {};

namespace sql {
class interface;
class iterator;
class query;

class row {
public:
  friend class iterator;
  row();
  row(row const &);
  ~row(void);
  row &operator=(row const &);
  bool operator==(row const &) const;
  bool operator!=(row const &) const;
  const std::string operator[](int) const;
  const std::string operator[](const std::string) const;
  int size() const;
protected:
  row(Row const &);
private:
  Row r;
};

class iterator {
public:
  friend class query;
  typedef input_iterator_tag iterator_category;
  typedef row                value_type;
  typedef ptrdiff_t          difference_type;
  typedef const row *        pointer;
  typedef const row &        reference;

  iterator();
  iterator(iterator const &);
  ~iterator(void);

  iterator &operator=(iterator &);
  bool      operator==(iterator &) const;
  bool      operator!=(iterator &) const;
  iterator &operator++(void);
  pointer   operator*() const;
  reference operator&() const;
  const std::string name(int) const;
private:
  iterator(ResUse);
  iterator(Query *);
  ResUse res;
  row r;
};

class query {
public:
  friend class iterator;
  query(query const &);
  query(interface &);
  ~query();
  query &operator<<(std::string);
#if 1
  iterator begin(void);
  iterator end(void);
#endif
  void exec(void);
  int  error(void) const;
private:
  std::string query_text;
  Connection *c;
  Query q;
  //  ResUse res;
  //  set<iterator *>activeits;
};

class interface  {
  friend class query;
public:
  interface();
  interface(interface const &);
  interface(const char *, const char *, const char *, const char *);
  ~interface(void);
  int error(void) const;
  void connect(const char *, const char *, const char *, const char *);
  interface &operator=(interface &);
  void lock(std::string s);
  void unlock(void);
  //  query &newquery();
private:
  Connection con;
  int err;
  //  set<query *> activequerys;
};


};
