/*********************************************************************
 *
 * Authors: Vincenzo Ciaschini - Vincenzo.Ciaschini@cnaf.infn.it 
 *
 * Copyright (c) 2002, 2003 INFN-CNAF on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or
 * http://www.edg.org/license.html
 *
 * Parts of this code may be based upon or even include verbatim pieces,
 * originally written by other people, in which case the original header
 * follows.
 *
 *********************************************************************/
#ifndef _SQLWRAP
#define _SQLWRAP

#include <iterator>
#include <string>

class DBEXC {};
class DBEXC_Connect : public DBEXC {};
class DBEXC_Query   : public DBEXC {};

class dbrow {
public:
  virtual ~dbrow(void) = 0;
  virtual bool operator==(dbrow const &) const = 0;
  virtual bool operator!=(dbrow const &) const = 0;
  virtual dbrow &operator=(dbrow const &) = 0;
  virtual const string operator[](int) const = 0;
  virtual const string operator[](const string) const = 0;
  virtual int size(void) const = 0;
  virtual const string name(int i) const = 0;

};

class dbiterator {
 public:
  typedef input_iterator_tag iterator_category;
  typedef dbrow              value_type;
  typedef ptrdiff_t          difference_type;
  typedef const dbrow *      pointer;
  typedef const dbrow &      reference;

  virtual bool operator==(dbiterator &) const = 0;
  virtual bool operator!=(dbiterator &) const = 0;
  virtual dbiterator &operator=(dbiterator &) = 0; 
  virtual ~dbiterator(void) = 0;
  virtual dbiterator &operator++(void) = 0;
  virtual pointer     operator*() const = 0;
  virtual reference   operator&() const = 0;
};

class dbquery {
 public:
  friend class dbiterator;

  virtual ~dbquery() = 0;
  virtual dbquery &operator<<(string) = 0;
  virtual dbiterator &start(void) = 0;
  virtual dbiterator &end(void) = 0;
  virtual void exec(void) = 0;
  virtual int  error(void) const = 0;
};

class dbinterface {
 public:
  virtual ~dbinterface(void) = 0;
  virtual void connect(const char *, const char *, const char *, const char *) = 0;
  virtual int error(void) const = 0;
  virtual dbquery &newquery() = 0;
  //  virtual dbinterface &operator=(dbinterface &) = 0;
};

#endif

