// TODO: Run multiple clients in parallel

#include "../../std.h"
#include <globus_common.h>
#include <globus_io.h>
#include <fstream>

#include "../../files/info_types.h"
#include "../../misc/log_time.h"
#include "../../misc/inttostring.h"
#include "../../misc/globus_error_utils.h"

#include "../client/client.h"
#include "../../misc/url_options.h"

#include "nl.nsmap"


class LoggerClient {
 private:
  URL url;
  HTTP_ClientSOAP* client;
  struct soap soap;
 public:
  LoggerClient(void);
  ~LoggerClient(void);
  bool Initialized(void);
  bool SameContact(const char* url);
  bool Report(const char* url,nl__jobinfo& info);
};


LoggerClient::LoggerClient(void):url(NULL),client(NULL) {
}

LoggerClient::~LoggerClient(void) {
  if(client) delete client;
}

bool LoggerClient::Initialized(void) {
  return (client != NULL);
}

bool LoggerClient::SameContact(const char* url_) {
  if(!url) return false;
  URL u(url_);
  return((u.Proto() == url.Proto()) && 
         (u.Host() == url.Host()) && 
         (u.Port() == url.Port()));
}

bool LoggerClient::Report(const char* url_,nl__jobinfo& info) {
  URL u(url_);
  if(url) {
    if((u.Proto() == url.Proto()) &&
       (u.Host() == url.Host()) &&
       (u.Port() == url.Port())) {
      url=u;
    } else { // reinitialize for different server
      url=u; if(client) delete client; client=NULL;
    };
  } else { // initialize for first connection
    url=u; if(client) delete client; client=NULL;
  };
  if(client == NULL) {
    string soap_url = url.Proto()+"://"+url.Host()+":"+inttostring(url.Port());
    client = new HTTP_ClientSOAP(soap_url.c_str(),&soap);
  };
  // connect to server if not yet connected
  if(client->connect() != 0) {
    odlog(0)<<"Failed to connect to "<<url<<endl;
    return -1;
  };
  // Send report
  string fake_soap_url = string("http://")+url.Host()+":"+inttostring(url.Port())+"/"+url.Path();
  int soap_err = SOAP_OK;
  int r;
  if((soap_err=soap_call_nl__add(&soap,fake_soap_url.c_str(),"add",&info,r)) != SOAP_OK) {
    odlog(0)<<"Failed to pass information to database"<<endl;
    r=1;
    if(LogTime::level > 0) soap_print_fault(&soap, stderr);
    client->disconnect();
  } else if(r != 0) {
    odlog(0)<<"Record refused. Error code: "<<r<<endl;
  };
  // Do not disconnect
  return (r==0);
}

#define MAX_CLIENTS 10
HTTP_ClientSOAP* clients[MAX_CLIENTS];


int logger(const char* url,char const * const * dirs,int num);

int main(int argc,char* argv[]) {
  char* url = "http://localhost:8000/logger";

  opterr=0;
  int n;
  while((n=getopt(argc,argv,":hc:l:u:d:")) != -1) {
    switch(n) {
      case ':': { olog<<"Missing argument\n"; return 1; };
      case '?': { olog<<"Unrecognized option\n"; return 1; };
      case 'h': {
        cout<<"logger [-u url] control_dir ..."<<endl;
         return 0;
      };
      case 'u': {
        url=optarg;
      }; break;
      case 'd': {
        char* p;
        int i = strtol(optarg,&p,10);
        if(((*p) != 0) || (i<0)) {
          olog<<"Improper debug level '"<<optarg<<"'"<<endl;
          exit(1);
        };
        LogTime::level=i;
      }; break;
      default: { olog<<"Options processing error\n"; return 1; };
    };
  };
  return logger(url,argv+optind,argc-optind);
}

int logger(const char* url,char const * const * dirs,int num) {
  int ret = 0;
  if(globus_module_activate(GLOBUS_IO_MODULE) != GLOBUS_SUCCESS) exit(1);

  {
  LoggerClient client;
  for(int n = 0;dirs[n] && n<num;n++) {
    string logdir(dirs[n]); logdir+="/logs/";
    struct dirent file_;
    struct dirent *file;
    odlog(1)<<"Processing directory: "<<logdir<<endl;
    DIR *dir=opendir(logdir.c_str());
    if(dir == NULL) continue;
    for(;;) {
      readdir_r(dir,&file_,&file);
      if(file == NULL) break;
      string logfile = logdir+file->d_name;
      struct stat st;
      if(stat(logfile.c_str(),&st) != 0) continue;
      if(!S_ISREG(st.st_mode)) continue;
      odlog(1)<<"Processing file: "<<logfile<<endl;
      /* read file */
      char buf[4096];
      ifstream f(logfile.c_str());
      if(!f.is_open() ) continue; /* can't open file */
      nl__jobinfo info;  // TODO: constructor/destructor
      info.start=0; info.end=0; info.cluster=NULL; info.user=NULL;
      info.id=NULL; info.name=NULL; info.failure=NULL; info.lrms=NULL;
      info.queue=NULL; info.rsl=NULL; info.ui=NULL;
      info.usedcpu=0; info.usedmem=0;
      char* job_url=NULL;
      for(;!f.eof();) {
        istream_readline(f,buf,sizeof(buf));
        char* value = strchr(buf,'=');
        if(!value) continue;
        char* name=buf; (*value)=0; value++;
        #define COMPARE_AND_SET(NM,ATTR) \
          if(!strcmp(name,NM)) { \
            if(info.ATTR) free(info.ATTR); \
            info.ATTR=strdup(value); continue; \
          }
        odlog(2)<<"name: "<<name<<" - value: "<<value<<endl;
        if(!strcmp(name,"loggerurl")) {
          if(job_url) free(job_url); job_url=strdup(value); continue; 
        };
        if(!strcmp(name,"starttime")) {
          mds_time t; t=value; if(t != (time_t)(-1)) info.start=t;
          continue; 
        };
        if(!strcmp(name,"endtime")) {
          mds_time t; t=value; if(t != (time_t)(-1)) info.end=t;
          continue; 
        };
        COMPARE_AND_SET("gridid",id);
        COMPARE_AND_SET("lrms",lrms);
        COMPARE_AND_SET("queue",queue);
        COMPARE_AND_SET("subject",user);
        COMPARE_AND_SET("jobname",name);
        COMPARE_AND_SET("failure",failure);
        COMPARE_AND_SET("rsl",rsl);
        COMPARE_AND_SET("clientname",ui);
      };
      f.close();
      if(job_url) url=job_url;
      odlog(2)<<"Reporting to: "<<url<<endl;
      bool result = client.Report(url,info);

      if(info.cluster) free(info.cluster);
      if(info.user)free(info.user);
      if(info.id) free(info.id);
      if(info.name) free(info.name);
      if(info.failure) free(info.failure);
      if(info.lrms) free(info.lrms);
      if(info.queue) free(info.queue);
      if(info.rsl) free(info.rsl);
      if(info.ui) free(info.ui);
      if(job_url) free(job_url);
      if(result) {
        unlink(logfile.c_str());
      } else {
        ret=-1;
      };
    };
    closedir(dir);
  };
  };
  globus_module_deactivate(GLOBUS_IO_MODULE);
  return ret;
}

