#include "../../std.h"
#include <iostream>
#include <fstream>

#include "../../misc/inttostring.h"
#include "../../misc/stringtoint.h"
#include "../../misc/escaped.h"
#include "../../misc/log_time.h"
#include "../../config/environment.h"
#include "../../auth/auth.h"



#undef SOAP_FMAC1
#undef SOAP_FMAC3
#undef SOAP_FMAC5
#undef SOAP_NMAC
#define WITH_NONAMESPACES
#define WITH_NOGLOBAL
#include "stdsoap2.h"
#undef SOAP_FMAC1
#undef SOAP_FMAC3
#undef SOAP_FMAC5
#undef SOAP_NMAC
#define SOAP_FMAC1 static
#define SOAP_FMAC3 static
#define SOAP_FMAC5 static
#define SOAP_NMAC static
#include "../logger.h"
#include "nl.nsmap"
#include "logger_soapC.cpp"
#include "logger_soapServer.cpp"


class acl_t {
 public:
  list<string> read;
  list<string> write;
  string sqluser;
  string sqlpassword;
};


HTTP_Service* logger_service_creator(HTTPS_Connector& c,const char* uri,void* arg) {
  acl_t* acl = (acl_t*)arg;
  /* check authorization */
  bool acl_read=false;
  bool acl_write=false;
  list<AuthEvaluator*>& auths = c.authorizations();
  for(list<AuthEvaluator*>::iterator ii=auths.begin();ii!=auths.end();++ii){
    if(!acl_read)
    for(list<string>::iterator i=acl->read.begin();i!=acl->read.end();++i) {
      if((*(*ii)) == (*i)) {
        if((*(*ii)).evaluate(c.identity()) == AAA_POSITIVE_MATCH) {
          odilog(1,c.pid)<<"User is granted 'read' access through acl '"<<(*(*ii)).get_name()<<"'"<<endl;
          acl_read=true;
        };
        break;
      };
    };
    if(!acl_write)
    for(list<string>::iterator i=acl->write.begin();i!=acl->write.end();++i) {
      if((*(*ii)) == (*i)) {
        if((*(*ii)).evaluate(c.identity()) == AAA_POSITIVE_MATCH) {
          odilog(1,c.pid)<<"User is granted 'write' access through acl '"<<(*(*ii)).get_name()<<"'"<<endl;
          acl_write=true;
        };
        break;
      };
    };
  };
  HTTP_Logger* h = new HTTP_Logger(&c,acl_read,acl_write,
              acl->sqluser.length()?acl->sqluser.c_str():NULL,
              acl->sqlpassword.length()?acl->sqlpassword.c_str():NULL);
  return h;
}

bool logger_service_configurator(istream& f,const char* uri,HTTP_Service_Properties &prop) {
  acl_t* acl = new acl_t;
  acl->sqluser="nglogger";
  for(;!f.eof();) {
    char buf[1024];
    istream_readline(f,buf,sizeof(buf));
    char* p = buf;
    for(;*p;p++) if(!isspace(*p)) break;
    char* command = p;
    if(*command == '#') continue;
    for(;*p;p++) if(isspace(*p)) break;
    int command_len = p-command;
    if(!command_len) continue;
    if((command_len==3) && (strncmp(command,"end",command_len)==0)) { break; }
    else if((command_len==8) && (strncmp(command,"acl_read",command_len)==0)) {
      for(;;) {
        string name;
        int n = input_escaped_string(p,name);
        if(n == 0) break;
        p+=n;
        acl->read.push_back(name);
      };
    } else if((command_len==9) && (strncmp(command,"acl_write",command_len)==0)) {
      for(;;) {
        string name;
        int n = input_escaped_string(p,name);
        if(n == 0) break;
        p+=n;
        acl->write.push_back(name);
      };
    } else if((command_len==10) && (strncmp(command,"sqlcontact",command_len)==0)) {
      int n;
      n=input_escaped_string(p,acl->sqluser);
      if(n) {
        p+=n;
        n=input_escaped_string(p,acl->sqlpassword);
      } else {
        acl->sqlpassword="";
      };
    } else if((command_len==16) && (strncmp(command,"sqlcontactsource",command_len)==0)) {
      string path;
      int n;
      n=input_escaped_string(p,path);
      acl->sqluser="";
      acl->sqlpassword="";
      if(n) {
        ifstream sf(path.c_str());
        if(sf) {
          char buf[1024];
          istream_readline(sf,buf,sizeof(buf));
          char* p = buf;
          n=input_escaped_string(p,acl->sqluser);
          if(n) {
            p+=n;
            n=input_escaped_string(p,acl->sqlpassword);
          };
        };
      };
    };
  };
  prop.arg=acl;
  prop.subtree=false;
  return true;
}

static time_t time2stamp(const char* s) {
  struct tm t;
  sscanf(s,"%u-%u-%u %u:%u:%u",&t.tm_year,&t.tm_mon,&t.tm_mday,&t.tm_hour,&t.tm_min,&t.tm_sec);
  t.tm_year-=1900; t.tm_mon--;
  time_t tl=mktime(&t);
  struct tm t2_;
  struct tm *t2;
  t2=gmtime_r(&tl,&t2_);
  time_t tl2=mktime(t2);
  tl=tl-(tl2-tl);
  return tl;
}

static string stamp2time(time_t t) {
  struct tm t2_;
  struct tm *t2;
  if(t==0) return string("EPOCH");
  if((t2=gmtime_r(&t,&t2_)) == NULL) return string("---"); 
  string s=inttostring(t2->tm_year + 1900)+"-"+
     inttostring(t2->tm_mon+1)+"-"+
     inttostring(t2->tm_mday)+" "+
     inttostring(t2->tm_hour)+":"+
     inttostring(t2->tm_min)+":"+
     inttostring(t2->tm_sec);
  return s;
} 

static void remove_hex(char* s) {
  char* p = s;
  char c[5];
  unsigned int i;
  c[0]='0'; c[1]='x'; c[4]=0;
  for(;*p;) {
    if((*p) == '%') {
      p++; if((*p) == 0) { (*s)='%'; s++; break; };
      c[2]=(*p); p++; if((*p) == 0) { (*s)='%'; s++; (*s)=c[2]; s++; break; };
      c[3]=(*p);
      if(sscanf(c,"%x",&i) != 1) {
        (*s)='%'; s++; (*s)=c[2]; s++; (*s)=c[3]; s++;
        p++;
        continue;
      };
      (*s)=(char)i; s++; p++; continue;
    };
    (*s)=(*p); if((*s) == '+') (*s)=' '; s++; p++;
  };
  (*s)=0;
};

/* callback functions */ 
// !!! TODO: remove them - use HTTP_ServiceAdv instead
int HTTP_Logger::local_fsend(struct soap *sp, const char* buf, size_t l) {
  HTTP_Logger *it = (HTTP_Logger*)(sp->user);
  if(it->ignore_output) return SOAP_OK; /* to hide gsoap response */
  if(it->c->write(buf,l)) return SOAP_SSL_ERROR; 
  return SOAP_OK;
}

size_t HTTP_Logger::local_frecv(struct soap* sp, char* buf, size_t l) {
  HTTP_Logger *it = (HTTP_Logger*)(sp->user);
  return it->c->read(buf,l);
}

static inline void as(string &s,const char* p) { if(p) s+=p; };

int HTTP_Logger::local_parse(struct soap *sp) {
  HTTP_Logger *it = (HTTP_Logger*)(sp->user);
  char header[SOAP_HDRLEN], *s;
  *sp->endpoint = '\0';
  sp->length = 0;
  /* read rest of header */
  for (;;) {
    if (soap_getline(sp,header,SOAP_HDRLEN)) return SOAP_EOF;
    if (!*header) break;
    s=strchr(header, ':');
    if (s) {
      *s = '\0'; do s++; while (*s && *s <= 32);
    };
    sp->fparsehdr(sp,header,s);
  };
  if(sp->keep_alive < 0) { sp->keep_alive=sp->keep_alive; }
  else if(sp->keep_alive == 1) { sp->keep_alive=0; }; /* HTTP/1.0 without keep_alive */
  /* TODO fill soap->path, soap->endpoint, etc. */
  if((sp->action) == NULL) {
    odilog(1,it->c->pid)<<"Looks like not SOAP request. Assuming plain HTTP POST."<<endl;
    char buf[1024];
    char* p = buf;
    int l,ll;
    l=sp->length;
    if((l==0) || (l>(sizeof(buf)-1))) {
      return SOAP_EOF;
    };
    for(;l>0;) {
      ll=local_frecv(sp,p,l);
      if(ll==0) return SOAP_EOF;
      l-=ll; p+=ll;
    };
    p=buf;
    char* p1;
    char* p2;
    char* query = NULL;
    unsigned long long offset = 0;
    bool show_start=false;
    bool show_end=false;
    bool show_cluster=false;
    bool show_user=false;
    bool show_name=false;
    bool show_id=false;
    bool show_failure=false;
    bool show_lrms=false;
    bool show_queue=false;
    bool show_ui=false;
    bool show_rsl=false;
    bool show_usedcpu=false;
    bool show_usedmem=false;
    for(;p;) {
      p1=strchr(p,'&'); if(p1) { (*p1)=0; p1++; };
      p2=strchr(p,'='); if(p2) (*p2)=0;
      if(p2) {
        p2++; remove_hex(p2);
        if(strcmp("query",p) == 0) {
          query=p2; p=p1; continue;
        };
        if(strcmp("offset",p) == 0) {
          stringtoint(p2,offset); p=p1; continue;
        };
        if(strcmp("show_start",p) == 0) {
          if(strcmp("on",p2) == 0) show_start=true; p=p1; continue;
        };
        if(strcmp("show_end",p) == 0) {
          if(strcmp("on",p2) == 0) show_end=true; p=p1; continue;
        };
        if(strcmp("show_cluster",p) == 0) {
          if(strcmp("on",p2) == 0) show_cluster=true; p=p1; continue;
        };
        if(strcmp("show_user",p) == 0) {
          if(strcmp("on",p2) == 0) show_user=true; p=p1; continue;
        };
        if(strcmp("show_name",p) == 0) {
          if(strcmp("on",p2) == 0) show_name=true; p=p1; continue;
        };
        if(strcmp("show_id",p) == 0) {
          if(strcmp("on",p2) == 0) show_id=true; p=p1; continue;
        };
        if(strcmp("show_failure",p) == 0) {
          if(strcmp("on",p2) == 0) show_lrms=true; p=p1; continue;
        };
        if(strcmp("show_queue",p) == 0) {
          if(strcmp("on",p2) == 0) show_queue=true; p=p1; continue;
        };
        if(strcmp("show_ui",p) == 0) {
          if(strcmp("on",p2) == 0) show_ui=true; p=p1; continue;
        };
        if(strcmp("show_rsl",p) == 0) {
          if(strcmp("on",p2) == 0) show_rsl=true; p=p1; continue;
        };
        if(strcmp("show_usedcpu",p) == 0) {
          if(strcmp("on",p2) == 0) show_usedcpu=true; p=p1; continue;
        };
        if(strcmp("show_usedmem",p) == 0) {
          if(strcmp("on",p2) == 0) show_usedmem=true; p=p1; continue;
        };
      };
      p=p1;
    }; 
    /* Process request */
    if(query == NULL) return SOAP_EOF;
    array_jobinfo r;
    // sp->keep_alive=0;
    if(nl__get(sp,query,offset,r) != SOAP_OK) {
       return SOAP_EOF;
    };
    /* Send response */
    HTTP_Logger *it = (HTTP_Logger*)(sp->user);
    HTTPS_Connector* c_ = it->c;
    string head_html = nordugrid_loc+"/share/logger_head.html";
    string tail_html = nordugrid_loc+"/share/logger_tail.html";
    if(send_file(head_html.c_str(),*c_)) return SOAP_EOF;
    string line;
    line="<TABLE BORDER=\"1\">\r\n";
    if(local_fsend(sp,line.c_str(),line.length()) != SOAP_OK) return SOAP_EOF;
    line="<TR><TD>#</TD>";
    if(show_start) line+="<TD>Start</TD>";
    if(show_end) line+="<TD>End</TD>";
    if(show_cluster) line+="<TD>Cluster</TD>";
    if(show_user) line+="<TD>Owner</TD>";
    if(show_name) line+="<TD>Name</TD>";
    if(show_id) line+="<TD>ID</TD>";
    if(show_failure) line+="<TD>Failed</TD>";
    if(show_lrms) line+="<TD>LRMS</TD>";
    if(show_queue) line+="<TD>Queue</TD>";
    if(show_ui) line+="<TD>Interface</TD>";
    if(show_rsl) line+="<TD>RSL</TD>";
    if(show_usedcpu) line+="<TD>CPU used</TD>";
    if(show_usedmem) line+="<TD>Mem. used</TD>";
    line+="</TR>";
    if(local_fsend(sp,line.c_str(),line.length()) != SOAP_OK) return SOAP_EOF;
    for(int n=0;n<r.__size;n++) {
      line="<TR><TD>"+inttostring(offset+n)+"</TD>";
      if(show_start){line+="<TD>";line+=stamp2time(r.__ptr[n].start);line+="</TD>";};
      if(show_end){line+="<TD>";line+=stamp2time(r.__ptr[n].end);line+="</TD>";};
      if(show_cluster){line+="<TD>";as(line,r.__ptr[n].cluster);line+="</TD>";};
      if(show_user){line+="<TD>";as(line,r.__ptr[n].user);line+="</TD>";};
      if(show_name){line+="<TD>";as(line,r.__ptr[n].name);line+="</TD>";};
      if(show_id){line+="<TD>";as(line,r.__ptr[n].id);line+="</TD>";};
      if(show_failure){line+="<TD>";as(line,r.__ptr[n].failure);line+="</TD>";};
      if(show_lrms){line+="<TD>";as(line,r.__ptr[n].lrms);line+="</TD>";};
      if(show_queue){line+="<TD>";as(line,r.__ptr[n].queue);line+="</TD>";};
      if(show_ui){line+="<TD>";as(line,r.__ptr[n].ui);line+="</TD>";};
      if(show_rsl){line+="<TD>";as(line,r.__ptr[n].rsl);line+="</TD>";};
      if(show_usedcpu){line+="<TD>-</TD>";};
      if(show_usedmem){line+="<TD>-</TD>";};
      line+="</TR>\r\n"; 
      if(local_fsend(sp,line.c_str(),line.length()) != SOAP_OK) return SOAP_EOF;
    };
    if(send_file(tail_html.c_str(),*c_)) return HTTP_ERROR;
    /* Disable gSOAP response and quit */
    it->ignore_output=true;
    it->plain_post=true;
    return SOAP_EOF;
  };
/*
  return soap_set_receiver_error(sp, "HTTP error", soap->msgbuf, SOAP_HTTP_ERROR);
*/
  return SOAP_OK;
}

HTTP_Logger::HTTP_Logger(HTTPS_Connector *c_,bool acl_read_,bool acl_write_,const char* user,const char* password) {
  c=c_; acl_read=acl_read_; acl_write=acl_write_;
  ignore_output=false;
  /* SOAP */
  soap_init(&sp);
  sp.namespaces=namespaces;
  sp.user=this;
  sp.fsend=&local_fsend;
  sp.frecv=&local_frecv;
  sp.fparse=&local_parse;
  sp.keep_alive=1;
  soap_set_imode(&sp,SOAP_IO_KEEPALIVE);
  soap_set_omode(&sp,SOAP_IO_KEEPALIVE);
  /* MySQL */
  mysql_init(&sql);
  if(mysql_real_connect(&sql,"localhost",user,password,"nglogger",0,NULL,0) == NULL) {
    odlog(0)<<"Faled to connect to MySQL server: "<<mysql_error(&sql)<<endl;
  };
}

HTTP_Logger::~HTTP_Logger(void) {
  soap_destroy(&sp);
  soap_end(&sp);
  soap_done(&sp);
  mysql_close(&sql);
}

/*
 * HTTP methods
 */
HTTP_Error HTTP_Logger::get(const char* uri,int &keep_alive) {
  if(!c) {
    skip_header(*c,keep_alive);
    return HTTP_NOT_IMPLEMENTED;
  };
  if(!acl_read) {
    skip_header(*c,keep_alive);
    return HTTP_NOT_ALLOWED;
  };
  if(skip_header(*c,keep_alive) != 0) return HTTP_ERROR;
  char* line;
  keep_alive=0;
  string head_html = nordugrid_loc+"/share/logger_head.html";
  string tail_html = nordugrid_loc+"/share/logger_tail.html";
  if(send_file(head_html.c_str(),*c)) return HTTP_ERROR;
  if(send_file(tail_html.c_str(),*c)) return HTTP_ERROR;
  return HTTP_OK;
}

HTTP_Error HTTP_Logger::put(const char* uri,int &keep_alive) {
  skip_header(*c,keep_alive);
  return HTTP_NOT_IMPLEMENTED;
}

HTTP_Error HTTP_Logger::post(const char* uri,int &keep_alive) {
  if(!c) {
    skip_header(*c,keep_alive);
    return HTTP_NOT_IMPLEMENTED;
  };
  plain_post=false;
  ignore_output=false;
  /* keep_alive is inverted by parseheader */
  sp.keep_alive=keep_alive; /* 0,1,2 */
//  soap_serve(&sp);
#ifdef OLD_GSOAP
  soap_begin(&sp);
  if(soap_begin_recv(&sp) || 
     soap_envelope_begin_in(&sp) || 
     soap_recv_header(&sp) || 
     soap_body_begin_in(&sp)) {
    if(plain_post) { keep_alive=0; return HTTP_OK; };
    soap_send_fault(&sp); keep_alive=0; return HTTP_ERROR;
  } else {
    sp.error = soap_serve_nl__add(&sp);
    if (sp.error == SOAP_NO_METHOD) {
    sp.error = soap_serve_nl__get(&sp);
    };
    if (sp.error) {
      soap_send_fault(&sp);
    };
  };
#else
  if(soap_begin_recv(&sp) ||
     soap_envelope_begin_in(&sp) ||
     soap_recv_header(&sp) ||
     soap_body_begin_in(&sp) ||
     soap_peek_element(&sp)) {
    if(plain_post) { keep_alive=0; return HTTP_OK; };
    soap_send_fault(&sp); keep_alive=0; return HTTP_ERROR;
  } else {
    if(!soap_match_tag(&sp,sp.tag,"nl:add")) soap_serve_nl__add(&sp);
    else if(!soap_match_tag(&sp,sp.tag,"ns:get")) soap_serve_nl__get(&sp);
    else sp.error = SOAP_NO_METHOD;
    if (sp.error) {
      soap_send_fault(&sp);
    };
  };
#endif
  if(sp.keep_alive==0) keep_alive=0;
  soap_destroy(&sp);  // delete temporary classes
  soap_end(&sp);      // delete temporary data 
  ignore_output=false;
  return HTTP_OK;
}

/*
 *  SOAP interface functions
 */
int nl__add(struct soap *sp, nl__jobinfo *info, int &r) {
  HTTP_Logger* it = (HTTP_Logger*)(sp->user);
  if(!(it->acl_write)) {
    odilog(0,it->c->pid)<<"Client has no write access"<<endl;
    return SOAP_FAULT;
  };
  struct tm t_;
  struct tm *t;

  r=1; // error by default

  if(info->start == 0) {
    odilog(0,it->c->pid)<<"Missing needed argument (start time)"<<endl;
    return SOAP_OK;
  };
/*  cluster is identified by subject
  if(info->cluster == NULL) {
    odilog(0,c->pid)<<"Missing needed argument (start time)"<<endl;
    return SOAP_OK;
  };
*/
  if(info->user == NULL) {
    odilog(0,it->c->pid)<<"Missing needed argument (user)"<<endl;
    return SOAP_OK;
  };
  if(info->id == NULL) {
    odilog(0,it->c->pid)<<"Missing needed argument (job id)"<<endl;
    return SOAP_OK;
  };
  if(mysql_real_query(&(it->sql),"LOCK TABLES jobs WRITE",22) != 0) {
    odilog(0,it->c->pid)<<"Failed to lock table: "<<mysql_error(&(it->sql))<<endl;
    return SOAP_SVR_FAULT;
  };
  string tt;
  tt = stamp2time(info->start);
  string query = "UPDATE jobs SET ";
  string set = "start='"+tt+"'";
  if(info->end) set+=" , end='"+stamp2time(info->end)+"'";
//  set+=string(" , cluster='")+info->cluster+"'";
  set+=string(" , cluster='")+it->c->identity_subject()+"'";
  set+=string(" , user='")+info->user+"'";
  set+=string(" , id='")+info->id+"'";
  if(info->name) set+=string(" , name='")+info->name+"'";
  if(info->failure) set+=string(" , failure='")+info->failure+"'";
  if(info->lrms) set+=string(" , lrms='")+info->lrms+"'";
  if(info->queue) set+=string(" , queue='")+info->queue+"'";
  if(info->rsl) set+=string(" , rsl='")+info->rsl+"'";
  if(info->ui) set+=string(" , ui='")+info->ui+"'";
//  query+=set+" WHERE cluster='"+info->cluster+
  query+=set+" WHERE cluster='"+it->c->identity_subject()+
               "' && user='"+info->user+
               "' && id='"+info->id+
               "' && start='"+tt+"'";
  if(mysql_real_query(&(it->sql),query.c_str(),query.length()) != 0) {
    odilog(0,it->c->pid)<<"Failed to query MySQL server: "<<query<<endl;
    odilog(0,it->c->pid)<<"MySQL reported error: "<<mysql_error(&(it->sql))<<endl;
    return SOAP_SVR_FAULT;
  };
  const char* sql_info = mysql_info(&(it->sql));
  if(sql_info==NULL) {
    odilog(0,it->c->pid)<<"Query did not return an information"<<endl;
    return SOAP_SVR_FAULT;
  };
  unsigned long long int matched, changed, warnings;
  if(sscanf(sql_info,"Rows matched: %Lu Changed: %Lu Warnings: %Lu",
                                          &matched,&changed,&warnings) != 3) {
    odilog(0,it->c->pid)<<"Query did not return an interpreatable information: "<<info<<endl;
    return SOAP_SVR_FAULT;
  };
  if(matched == 0) {
    query = "INSERT jobs SET "+set;
    if(mysql_real_query(&(it->sql),query.c_str(),query.length()) != 0) {
      odilog(0,it->c->pid)<<"Failed to query MySQL server: "<<query<<endl;
      odilog(0,it->c->pid)<<"MySQL reported error: "<<mysql_error(&(it->sql))<<endl;
      return SOAP_SVR_FAULT;
    };
  };
  if(mysql_real_query(&(it->sql),"UNLOCK TABLES",13) != 0) {
    odilog(0,it->c->pid)<<"Warning: Failed to unlock table: "<<mysql_error(&(it->sql))<<endl;
  };
  r=0;
  return SOAP_OK;
}

static int mysql_field_num(const MYSQL_FIELD* fields,unsigned int num_fields,const char* name) {
  unsigned int num = 0;
  for(;num<num_fields;num++) {
    if(strcmp(fields[num].name,name)==0) break;
  };
  if(num>=num_fields) return -1;
  return num;
}

static int mysql_get_datetime(MYSQL_ROW sql_row,int n,time_t* t) {
  char* value;
  if(n>=0) {
    value=sql_row[n];
    if(value) {
      (*t)=time2stamp(value);
      return 0;
    };
  };
  (*t)=0;
  return 1;
}

static int mysql_get_string(const MYSQL_ROW sql_row,int n,char** s) {
  char* value;
  if(n>=0) {
    value=sql_row[n];
    if(value) {
      (*s)=strdup(value);
      return 0;
    };
  };
  (*s)=NULL;
  return 1;
}

int mysql_get_int(const MYSQL_ROW sql_row,int n,int* i) {
  char* value;
  if(n>=0) {
    value=sql_row[n];
    if(value) {
      char* e;
      (*i)=strtol(value,&e,0);
      if((*e) == 0) return 0;
    };
  };
  (*i)=0;
  return 1;
}

int nl__get(struct soap* sp, char *query, ULONG64 offset, array_jobinfo &r) {
  HTTP_Logger* it = (HTTP_Logger*)(sp->user);
  if(!(it->acl_read)) return SOAP_FAULT;
  string q("SELECT * FROM jobs");
  if((query != NULL) && ((*query) != 0)) { q+=" WHERE "; q+=query; };
  q+=" ORDER BY start LIMIT "; q+=inttostring(offset)+",100";
  if(mysql_real_query(&(it->sql),q.c_str(),q.length()) != 0) {
    odilog(0,it->c->pid)<<"Failed to query MySQL server with: "<<q<<endl;
    odilog(0,it->c->pid)<<"MySQL error: "<<mysql_error(&(it->sql))<<endl;
    return SOAP_SVR_FAULT;
  };
  MYSQL_RES* sql_res;
  if((sql_res=mysql_use_result(&(it->sql))) == NULL) {
    odilog(0,it->c->pid)<<"Failed to initiate retrieving results from MySQL server: "<<mysql_error(&(it->sql))<<endl;
    return SOAP_SVR_FAULT;
  }; 
  unsigned int num_fields = mysql_num_fields(sql_res);
  MYSQL_FIELD* fields = mysql_fetch_fields(sql_res);
  unsigned int start_num = mysql_field_num(fields,num_fields,"start");
  unsigned int end_num = mysql_field_num(fields,num_fields,"end");
  unsigned int cluster_num = mysql_field_num(fields,num_fields,"cluster");
  unsigned int id_num = mysql_field_num(fields,num_fields,"id");
  unsigned int user_num = mysql_field_num(fields,num_fields,"user");
  unsigned int name_num = mysql_field_num(fields,num_fields,"name");
  unsigned int failure_num = mysql_field_num(fields,num_fields,"failure");
  unsigned int lrms_num = mysql_field_num(fields,num_fields,"lrms");
  unsigned int queue_num = mysql_field_num(fields,num_fields,"queue");
  unsigned int rsl_num = mysql_field_num(fields,num_fields,"rsl");
  unsigned int ui_num = mysql_field_num(fields,num_fields,"ui");
  unsigned int usedcpu_num = mysql_field_num(fields,num_fields,"usedcpu");
  unsigned int usedmem_num = mysql_field_num(fields,num_fields,"usedmem");
  int count = 0;
  r.__ptr = new nl__jobinfo[100];
  for(;;) {
    if(count >= 100) break;
    MYSQL_ROW sql_row;
    if((sql_row=mysql_fetch_row(sql_res)) == NULL) break;
    mysql_get_datetime(sql_row,start_num,&(r.__ptr[count].start));
    mysql_get_datetime(sql_row,end_num,&(r.__ptr[count].end));
    mysql_get_string(sql_row,cluster_num,&(r.__ptr[count].cluster));
    mysql_get_string(sql_row,user_num,&(r.__ptr[count].user));
    mysql_get_string(sql_row,id_num,&(r.__ptr[count].id));
    mysql_get_string(sql_row,name_num,&(r.__ptr[count].name));
    mysql_get_string(sql_row,failure_num,&(r.__ptr[count].failure));
    mysql_get_string(sql_row,lrms_num,&(r.__ptr[count].lrms));
    mysql_get_string(sql_row,queue_num,&(r.__ptr[count].queue));
    mysql_get_string(sql_row,rsl_num,&(r.__ptr[count].rsl));
    mysql_get_string(sql_row,ui_num,&(r.__ptr[count].ui));
    mysql_get_int(sql_row,usedcpu_num,&(r.__ptr[count].usedcpu));
    mysql_get_int(sql_row,usedmem_num,&(r.__ptr[count].usedmem));
    count++;
  };
  r.__size=count;
  mysql_free_result(sql_res);
  return SOAP_OK;
}

